/*
 * Ambient - the ultimate desktop
 * ------------------------------
 * Copyright 2001-2005 by David Gerber <zapek@morphos.net>
 * Copyright 2005-2006 Ambient Open Source Team
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * $Id: panelseparatorclass.c,v 1.1 2006/12/20 13:34:15 fab Exp $
 */

#include "ambient.h"

/* public */

/* private */
#include "mui_func.h"
#include "rexx.h"
#include "command.h"
#include "contextmenu.h"
#include "name.h"
#include "prefs.h"
#include "threads.h"
#include "file_func.h"
#include "legacy.h"
#include "prefs_advanced.h"

struct Data {
	ULONG  distance;
	APTR   cmenu;
};


static void change_direction(APTR obj, ULONG horiz)
{
	if (horiz)
	{
		SetAttrs(obj,
			MUIA_Rectangle_VBar, TRUE,
			MUIA_Rectangle_HBar, FALSE,
		TAG_DONE);
	}
	else	
	{
		SetAttrs(obj,
			MUIA_Rectangle_VBar, FALSE,
			MUIA_Rectangle_HBar, TRUE,
		TAG_DONE);				
	}
}

static void doset(APTR obj, struct Data *data, struct TagItem *tags)
{
	ULONG update = 0;

	FORTAG(tags)
	{
		case MA_Panel_Distance:
			data->distance = tag->ti_Data;
			break;

		case MA_Panelgroup_Horiz:
			/*  we don't get new MUIM_Setup on direction change in PanelPrefs, so this is required
			 */
			change_direction(obj, tag->ti_Data);
			break;
	}
	NEXTTAG 
}


DEFNEW
{
	obj = DoSuperNew(cl, obj,
		MUIA_Frame, MUIV_Frame_None,
		#if 0
		MUIA_Rectangle_VBar, TRUE,  /* XXX: needs to be set properly here somehow... but no info if panel is horiz/vert in INITTAGS */
		#endif
		MUIA_InnerLeft,   2,
		MUIA_InnerRight,  2, 
		MUIA_InnerTop,    2, 
		MUIA_InnerBottom, 2,
		MUIA_ContextMenu, 1,
		TAG_DONE
	);

	if (obj)
	{
		GETDATA;
	
		doset(obj, data, INITTAGS);	
	}  

	return ((ULONG)obj);
}



DEFMMETHOD(AskMinMax)
{
	GETDATA;

	APTR  parent = _parent(obj);
	ULONG size   = getv(_parent(obj), MA_Panelgroup_Size);
	ULONG horiz  = getv(_parent(obj), MA_Panelgroup_Horiz);
	
	ULONG defsize = 12; /* XXX: make configurable in advanced prefs? */

	DOSUPER;


	if (horiz)
	{
		if (msg->MinMaxInfo->MinWidth < defsize)
		{
			msg->MinMaxInfo->MinWidth = defsize;
			msg->MinMaxInfo->MaxWidth = defsize;
		}
		msg->MinMaxInfo->MinHeight = size;
		msg->MinMaxInfo->MaxHeight = size;
	}
	else
	{
		if (msg->MinMaxInfo->MinHeight < defsize)
		{
			msg->MinMaxInfo->MinHeight = defsize;
			msg->MinMaxInfo->MaxHeight = defsize;
		}

		msg->MinMaxInfo->MinWidth = size;
		msg->MinMaxInfo->MaxWidth = size;
	}

	return 0;
}


DEFGET
{
	GETDATA;

	switch (msg->opg_AttrID)
	{
		case MA_Panel_Type:
			*msg->opg_Storage = MV_Panel_Type_Separator;
			return (TRUE);
			break;

		case MA_Panel_Distance:
			*msg->opg_Storage = data->distance;
			return (TRUE);
			break;

		case MA_Panel_Properties:
            *msg->opg_Storage = TRUE;
			return (TRUE);
			break;
	}

	return DOSUPER;
}


DEFSET
{
	GETDATA;

	doset(obj, data, INITTAGS);

	return (DOSUPER);
}


DEFMMETHOD(Setup)
{	
	change_direction(obj, getv(_parent(obj), MA_Panelgroup_Horiz));

	return DOSUPER;
}


DEFMMETHOD(ContextMenuAdd)
{
	ULONG flags = 0;
	GETDATA;

	if (!getv(_parent(obj), MA_Panelgroup_Locked))
	{
		flags |= AS_PANEL_UNLOCKED;
	}

	data->cmenu	= contextmenu_build_simple(CM_PANELSEPARATOR, flags);

	if (data->cmenu)
	{
		DoMethod(msg->menustrip, OM_ADDMEMBER, data->cmenu);
	}

	return (DOSUPER);
}


DEFMMETHOD(ContextMenuChoice)
{
	GETDATA;
	struct command_menu *cm;
	ULONG rc = 0;

	if (data->cmenu && DoMethod(data->cmenu, MUIM_FindObject, msg->item))
	{
		if ( ((cm = (struct command_menu *)getv(msg->item, MA_Menuitem_Command))) )
		{
			if (cm->name && *cm->name)
				execute_command(_parent(obj), cm->type, cm->args, NULL);
		}
	}
	else
		rc = DOSUPER;

	data->cmenu = NULL;

	return rc;
}


DEFSMETHOD(Panel_SaveConfig)
{
	APTR pl, pi; /* list, item */
	GETDATA;
	ASSERT(msg->pctx);

	if (!(pl = prefspool_item_get(msg->pctx, NULL, DSI_LISTPOOL_PANEL, NULL, NULL)))
	{
		pl = prefspool_item_add(msg->pctx, NULL, DSI_LISTPOOL_PANEL, NULL, NULL);
	}

	if (pl)
	{
		if (!(pi = prefspool_item_get(msg->pctx, pl, msg->index | DSF_LISTPOOL, NULL, NULL)))
		{
			pi = prefspool_item_add(msg->pctx, pl, msg->index | DSF_LISTPOOL, NULL, NULL);
		}
		if (pi)
		{
			setprefslong_lp(msg->pctx, pi, DSI_LISTPOOL_PANEL_DISTANCE, data->distance);
		}
	}
	
	return 0;
}


BEGINMTABLE
DECNEW
DECGET
DECSET
DECMMETHOD(AskMinMax)
DECMMETHOD(Setup)
DECMMETHOD(ContextMenuChoice)
DECMMETHOD(ContextMenuAdd)
DECSMETHOD(Panel_SaveConfig)
ENDMTABLE

DECSUBCLASS_NC(MUIC_Rectangle, panelseparatorclass)

