#include "ambient.h"

#if USE_TRAY_LIB

/* public */
#include <exec/lists.h>

#define USE_INLINE_STDARG 1

#include <proto/tray.h>

/* private */
#include "mui_func.h"
#include "methodstack.h"
#include "traylib.h"

static APTR find_panelgroup(void);

typedef struct WrapItem
{
	struct MinNode n;
	APTR wrapobject;
	APTR trayobject;
} WrapItem;

static tray_server *ts;
static struct MinList list;

/* API */

MUI_HOOK(tray_reg, tray_client *client, struct BitMap *bitmap)
{
	DB(("reg hook: bitmap %p\n", bitmap));


	return 0;
}


static int detach_trayobj(APTR wrapobj, APTR trayobj)
{
	if (methodstack_push_sync_safe(wrapobj, 1, MUIM_Group_InitChange))
	{
		methodstack_push_sync_safe(wrapobj, 2, OM_REMMEMBER, (ULONG)trayobj);

		methodstack_push_sync_safe(wrapobj, 1, MUIM_Group_ExitChange);

		return (TRUE);
	}

	return (FALSE);
}

static int detach_wrapobj(APTR wrapobj)
{
	APTR group = find_panelgroup();
	
    if (group)
	{
		if (methodstack_push_sync_safe(group, 1, MUIM_Group_InitChange))
		{
			methodstack_push_sync_safe(group, 2, OM_REMMEMBER, (ULONG)wrapobj);

			methodstack_push_sync_safe(group, 1, MUIM_Group_ExitChange);

			return (TRUE);
		}
	}

	return (FALSE);
}

MUI_HOOK(tray_unreg, tray_client *client, LONG *dummy)
{
	Object **objects;

	DB(("unreg hook\n"));

	tray_server_getclientdata(client, TRAY_CDATA_APPLETLIST, (ULONG)&objects, TAG_DONE);

	if (objects[0])
	{
		DB(("calling app:DetachTrayObject\n"));
		methodstack_push_sync_safe(app, 2, MM_Application_DetachTrayObject, objects[0]);
	}
	
    return 0;
}


static APTR find_panelgroup(void)
{
	APTR panelwin;

	panelwin = (APTR)methodstack_push_sync_safe(app, 2, MM_Application_FindWindowByType, MV_Window_Type_Panel);

	if (panelwin)
	{
		APTR panelgrp = NULL;

		methodstack_push_sync_safe(panelwin, 3, OM_GET, MA_Panelwin_Group, &panelgrp);

		return panelgrp;
    }

	return NULL;
}


MUI_HOOK(tray_applet, tray_client *client, Object **objects)
{
	STRPTR name;
	DB(("applet hook\n"));
	
	tray_server_getclientdata(client, TRAY_CDATA_NAME, (ULONG)&name, TAG_DONE);
	methodstack_push_sync_safe(app, 3, MM_Application_AttachTrayObject, objects[0], name);

	DB(("name %s\n", name));

//	  tray_client_pushmethod(client, objects[0], MUIM_Set, MUIA_Draggable, TRUE, TAG_DONE);

	set(objects[0], MUIA_Draggable, TRUE);
	return 0;
}

MUI_HOOK(tray_menu, tray_client *client, Object *menu)
{
	DB(("menu hook\n"));

	return 0;
}

void traylib_registerhook(void)
{
	if (ts)
	{
		tray_server_applet(ts, &tray_applet_hook, 0);
		tray_server_menu(ts, &tray_menu_hook);
    }
}

ULONG traylib_init(void)
{
	ts = NULL;

	if (TrayBase)
		ts = tray_server_register("ambient", &tray_reg_hook, &tray_unreg_hook);

	NEWLIST(&list);

	return TRUE;
}

void traylib_cleanup(void)
{
	if (ts)
		tray_server_unregister(ts);

}


#endif
